package main

import (
	"encoding/json"
	"fmt"
	"log"
	"net/http"
	"time"

	"github.com/gorilla/mux"
	"github.com/gorilla/sessions"
)

// Config 구조체 정의
type Config struct {
	ServerAddress string `json:"serverAddress"`
	RetryLimit    int    `json:"retryLimit"`
	RetryDelay    int    `json:"retryDelay"`
	LogFilePath   string `json:"logFilePath"`
	MaxLogSize    int64  `json:"maxLogSize"`
	ApiEndpoint   string `json:"apiEndpoint"`
}

var config Config

// 세션 저장소
var store = sessions.NewCookieStore([]byte("your-secret-key"))

func main() {
	// 초기 설정 파일 읽기
	configJSON := `
	{
	    "serverAddress": "localhost:8080",
	    "retryLimit": 5,
	    "retryDelay": 2000,
	    "logFilePath": "app.log",
	    "maxLogSize": 10485760,
	    "apiEndpoint": "http://example.com/api/logs"
	}`
	err := json.Unmarshal([]byte(configJSON), &config)
	if err != nil {
		log.Fatalf("Failed to unmarshal config JSON: %v", err)
	}

	r := mux.NewRouter()

	// 정적 파일 서빙
	r.PathPrefix("/static/").Handler(http.StripPrefix("/static/", http.FileServer(http.Dir("."))))

	// 로그인 페이지
	r.HandleFunc("/", serveLogin).Methods("GET")
	r.HandleFunc("/login", loginHandler).Methods("POST")

	// 보호된 페이지
	editSubrouter := r.PathPrefix("/edit").Subrouter()
	editSubrouter.HandleFunc("/", serveEdit).Methods("GET")
	editSubrouter.HandleFunc("/config", getConfig).Methods("GET")
	editSubrouter.HandleFunc("/save", saveConfig).Methods("POST")
	editSubrouter.HandleFunc("/logout", logoutHandler).Methods("GET")

	fmt.Println("Server listening on localhost:8000...")
	log.Fatal(http.ListenAndServe(":8000", r))
}

// 로그인 페이지 서빙
func serveLogin(w http.ResponseWriter, r *http.Request) {
	http.ServeFile(w, r, "index.html")
}

// 로그인 처리
func loginHandler(w http.ResponseWriter, r *http.Request) {
	username := r.FormValue("username")
	password := r.FormValue("password")

	// 실제로는 여기에서 데이터베이스나 다른 인증 방식을 사용하여 검증해야 합니다.
	if username == "admin" && password == "password" {
		session, _ := store.Get(r, "session-name")
		session.Values["authenticated"] = true
		session.Save(r, w)
		http.Redirect(w, r, "/edit", http.StatusSeeOther)
	} else {
		http.Error(w, "Invalid username or password", http.StatusUnauthorized)
	}
}

// 로그아웃 처리
func logoutHandler(w http.ResponseWriter, r *http.Request) {
	session, _ := store.Get(r, "session-name")
	session.Values["authenticated"] = false
	session.Save(r, w)
	http.Redirect(w, r, "/", http.StatusSeeOther)
}

// 보호된 페이지 서빙
func serveEdit(w http.ResponseWriter, r *http.Request) {
	session, _ := store.Get(r, "session-name")
	val := session.Values["authenticated"]
	if val == nil || !val.(bool) {
		http.Redirect(w, r, "/", http.StatusSeeOther)
		return
	}

	http.ServeFile(w, r, "index.html")
}

// 현재 설정을 제공하는 핸들러
func getConfig(w http.ResponseWriter, r *http.Request) {
	w.Header().Set("Content-Type", "application/json")
	json.NewEncoder(w).Encode(config)
}

// 설정을 업데이트하는 핸들러
func saveConfig(w http.ResponseWriter, r *http.Request) {
	session, _ := store.Get(r, "session-name")
	val := session.Values["authenticated"]
	if val == nil || !val.(bool) {
		http.Error(w, "Unauthorized", http.StatusUnauthorized)
		return
	}

	if r.Method != http.MethodPost {
		http.Error(w, "Method not allowed", http.StatusMethodNotAllowed)
		return
	}

	var newConfig Config
	err := json.NewDecoder(r.Body).Decode(&newConfig)
	if err != nil {
		http.Error(w, "Failed to decode JSON", http.StatusBadRequest)
		return
	}

	config = newConfig

	w.WriteHeader(http.StatusOK)
	fmt.Fprintf(w, "Configuration updated successfully")
}

